@@ -88,27 +88,29 @@ struct Command {
 
 
 fn print_help() {
-    print!("Input is interpreted as hex unless toggled to decimal with 'x'
+    print!("Input/output is hex unless toggled to decimal with 'x'
 ?          This help
 <Enter>    Print current byte(s) and move forward to next set of byte(s)
-314        Move to byte number 0x314 (or 0d314 depending on 'x') and print from
-               there
+3d4        Move to byte number 3d4 and print from there
 +          Move 1 byte forward and print from there
 -          Move 1 byte back and print from there
-+314       Move 0x314 bytes forward (or 0d314 depending on 'x') and print from
-               there
--314       Move 0x314 bytes back (or 0d314 depending on 'x') and print from
-               there
++3d4       Move 3d4 bytes forward and print from there
+-3d4       Move 3d4 bytes back and print from there
 $          Move to last byte and print it
+k          Delete (kill) byte at current index and print new line of byte(s)
+7dk        Move to byte 7d, delete that byte, and print from there.
+1d,72k     Move to byte 1d, delete bytes 1d - 72 inclusive, and print from there.
 i          Prompt you to write out bytes which will be inserted at current index
-12,34p     Print bytes 12 - 34 inclusive (depending on 'x'), then move to
-                leftmost byte printed on the last line.
+72i        Move to byte number 72 and prompt you to enter bytes which will be
+             inserted there.
+12,3dp     Print bytes 12 - 3d inclusive, move to leftmost byte printed on the
+             last line.
 n          Toggle whether or not byte numbers are printed before bytes
-p          Print current byte(s) (depending on 'W')
+p          Print current line of byte(s) (depending on 'W')
 s          Print state of all toggles and 'W'idth
 x          Toggle interpreting inputs and displaying output as hex or decimal
-W30        Print a linebreak every 0x30 bytes (or 0d30 bytes depending on 'x')
-               [Default 0x10]
+w          Actually write changes to the file on disk
+W3d        Print a linebreak every 3d bytes [Default 0x10]
 q          quit
 ");
 }
@@ -156,7 +158,7 @@ impl Command {
         let re_blank_line = Regex::new(r"^ *$").unwrap();
         let re_plus = Regex::new(r"^ *\+ *$").unwrap();
         let re_minus = Regex::new(r"^ *\- *$").unwrap();
-        let re_single_char_command = Regex::new(r"^ *(?P<command>[?npsxqi]).*$").unwrap();
+        let re_single_char_command = Regex::new(r"^ *(?P<command>[?npsxqwik]).*$").unwrap();
         let re_range = Regex::new(r"^ *(?P<begin>[0-9a-fA-F.$]+) *, *(?P<end>[0-9a-fA-F.$]+) *(?P<the_rest>.*) *$").unwrap();
         let re_specified_index = Regex::new(r"^ *(?P<index>[0-9A-Fa-f.$]+) *(?P<the_rest>.*) *$").unwrap();
         let re_offset_index = Regex::new(r"^ *(?P<sign>[-+])(?P<offset>[0-9A-Fa-f]+) *(?P<the_rest>.*) *$").unwrap();
@@ -220,14 +222,14 @@ impl Command {
             let command = caps.unwrap().name("command").unwrap().as_str().chars().next().unwrap();
             if command == 'p' {
                 Ok(Command{
-                    range: (0, 0),
+                    range: (index, index),
                     command: 'Q',
                     args: vec![],
                 })
             }
             else {
                 Ok(Command{
-                    range: (0, 0),
+                    range: (index, index),
                     command: command,
                     args: vec![],
                 })
@@ -608,8 +610,15 @@ fn print_state(state:&State) {
 
 /// returns index of the byte in the 0-th column of the last row printed
 fn print_bytes(state:&State, range:(usize, usize)) -> Option<usize> {
+    let max = max_index(&state);
+    if max.is_err() {
+        println!("? ({:?})", max);
+        return None;
+    }
+    let max = max.unwrap();
+
     let from = range.0;
-    let to = min(max_index(&state), range.1);
+    let to = min(max, range.1);
     if bad_range(&state.all_bytes, (from, to)) {
       println!("? (Bad range: ({}, {}))", range.0, range.1);
       return None;
@@ -687,6 +696,7 @@ struct State {
     radix: u32,
     show_byte_numbers: bool,
     unsaved_changes: bool,
+    filename: String,
 
     /* Current byte number, 0 to len -1 */
     index: usize,
@@ -703,9 +713,9 @@ struct State {
 
 impl fmt::Debug for State {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "radix: {}|unsaved_changes: {}|show_byte_numbers: {}|index: {}|width: {}|n_padding: '{}'|",
+        write!(f, "radix: {}|unsaved_changes: {}|show_byte_numbers: {}|index: {}|width: {}|n_padding: '{}'|filename: {}|",
                 self.radix, self.unsaved_changes, self.show_byte_numbers,
-                self.index, self.width, self.n_padding)
+                self.index, self.width, self.n_padding, self.filename)
     }
 }
 
@@ -715,8 +725,13 @@ fn range(state:&State) -> (usize, usize) {
 }
 
 
-fn max_index(state:&State) -> usize {
-    state.all_bytes.len() - 1
+fn max_index(state:&State) -> Result<usize, String> {
+    if state.all_bytes.len() == 0 {
+        Err("No bytes, so no max index.".to_owned())
+    }
+    else {
+        Ok(state.all_bytes.len() - 1)
+    }
 }
 
 
@@ -760,6 +775,7 @@ pub fn actual_runtime(filename: &str) -> i32 {
     // TODO Below here should be a function called main_loop()
     let mut state = State{
         radix: 16,
+        filename: filename.to_owned(),
         show_byte_numbers: true,
         unsaved_changes: false,
         index: 0,
@@ -785,8 +801,16 @@ pub fn actual_runtime(filename: &str) -> i32 {
             }
         };
 
+        // Do this idiomatically
+        let max = match max_index(&state) {
+            Ok(max) => max,
+            Err(_) => {
+                continue
+            },
+        };
+
         if let Ok(command) = Command::from_index_and_line(state.index, &input,
-                state.all_bytes.len() - 1, state.radix) {
+                max, state.radix) {
             // println!("{:?}", command);
             match command.command {
 
@@ -808,15 +832,22 @@ pub fn actual_runtime(filename: &str) -> i32 {
 
                 /* + */
                 'G' => {
-                    if state.index == max_index(&state) {
-                        println!("? (already at last byte");
-                    }
-                    else if state.index > max_index(&state) {
-                        println!("? (past last byte");
-                    }
-                    else {
-                        state.index += 1;
-                        print_bytes(&state, range(&state));
+                    match max_index(&state) {
+                        Ok(max) => {
+                            if state.index == max {
+                                println!("? (already at last byte");
+                            }
+                            else if state.index > max {
+                                println!("? (past last byte");
+                            }
+                            else {
+                                state.index += 1;
+                                print_bytes(&state, range(&state));
+                            }
+                        },
+                        Err(error) => {
+                            println!("? ({})", error);
+                        },
                     }
                 },
 
@@ -835,8 +866,10 @@ pub fn actual_runtime(filename: &str) -> i32 {
                 'i' => {
                     match read_bytes_from_user() {
                         Ok(entered_bytes) => {
+                            state.index = command.range.1;
                             // TODO Find the cheapest way to do this (maybe
                             // make state.all_bytes a better container)
+                            // TODO Do this with split_off
                             let mut new = Vec::with_capacity(state.all_bytes.len() + entered_bytes.len());
                             for i in 0..state.index {
                                 new.push(state.all_bytes[i]);
@@ -850,6 +883,7 @@ pub fn actual_runtime(filename: &str) -> i32 {
                             }
                             state.all_bytes = new;
                             state.unsaved_changes = true;
+                            print_bytes(&state, range(&state));
                         },
                         Err(error) => {
                             println!("? ({})", error);
@@ -862,6 +896,17 @@ pub fn actual_runtime(filename: &str) -> i32 {
                     print_help();
                 },
 
+                /* 'k'ill byte(s) (Can't use 'd' because that's a hex
+                 * character! */
+                'k' => {
+                    skip_bad_range!(command, state.all_bytes);
+                    let mut right_half = state.all_bytes.split_off(command.range.0);
+                    right_half = right_half.split_off(command.range.1 - command.range.0 + 1);
+                    state.all_bytes.append(&mut right_half);
+                    state.index = command.range.0;
+                    print_bytes(&state, range(&state));
+                },
+
                 /* Toggle showing byte number */
                 'n' => {
                     state.show_byte_numbers = !state.show_byte_numbers;
@@ -880,20 +925,27 @@ pub fn actual_runtime(filename: &str) -> i32 {
 
                 /* User pressed enter */
                 '\n' => {
-                    let width = usize::from(state.width);
-                    let first_byte_to_show_index = state.index + width;
-                    let last_byte_to_show_index = min(
-                            first_byte_to_show_index + width - 1,
-                                    max_index(&state));
-                    if first_byte_to_show_index > max_index(&state) {
-                        println!("? (already showing last byte at index {})",
-                                hex_unless_dec(last_byte_to_show_index,
-                                        state.radix));
-                    }
-                    else {
-                        state.index = first_byte_to_show_index;
-                        print_bytes(&state, (first_byte_to_show_index,
-                                last_byte_to_show_index));
+                    match max_index(&state) {
+                        Ok(max) => {
+                            let width = usize::from(state.width);
+                            let first_byte_to_show_index = state.index + width;
+                            let last_byte_to_show_index = min(
+                                    first_byte_to_show_index + width - 1,
+                                            max);
+                            if first_byte_to_show_index > max {
+                                println!("? (already showing last byte at index {})",
+                                        hex_unless_dec(last_byte_to_show_index,
+                                                state.radix));
+                            }
+                            else {
+                                state.index = first_byte_to_show_index;
+                                print_bytes(&state, (first_byte_to_show_index,
+                                        last_byte_to_show_index));
+                            }
+                        },
+                        Err(error) => {
+                            println!("? ({})", error);
+                        }
                     }
                 }
 
@@ -936,6 +988,14 @@ pub fn actual_runtime(filename: &str) -> i32 {
                     print_state(&state);
                 },
 
+                /* Write out */
+                'w' => {
+                    let result = std::fs::write(filename, &state.all_bytes);
+                    if result.is_err() {
+                        println!("? (Couldn't write to {})", state.filename);
+                    }
+                },
+
                 /* Change width */
                 'W' => {
                     if let Some(width) = NonZeroUsize::new(command.range.0) {
